from .config import *

class DialogBox(pygame.sprite.DirtySprite):
    def __init__(self, parent):
        pygame.sprite.DirtySprite.__init__(self, global_var.ALL_SPRITES)

        self.dirty = 2
        self.orig_image = pygame.image.load(cwd+'\xiaobanma\DialogBox.png')
        self.font = pygame.font.Font(cwd+'\xiaobanma\simkai.ttf', 25)
        self._text = ''
        self._parent = parent
        self.hide()

    def say(self, text = None):
        if text:
            self._text = str(text)
        else:
            self.hide()

    def show(self):
        self._visible = True

    def hide(self):
        self._visible = False


    def update(self):
        self.image = self.orig_image.copy()
        if self._parent.pos[0] > 0:
            flipx = True
        else:
            flipx = False
        
        if flipx:
            self.image = pygame.transform.flip(self.image, flipx, 0)
        lines = len(self._text)//12+1
        self.image = pygame.transform.scale(self.image, (int(self.image.get_width() * 0.6), int(self.image.get_height() * lines / 10 + 50)))
        for i in range(lines):
            text = self.font.render(f"{self._text[i*12:(i+1)*12]}", 1, (0, 0, 0), (255, 255, 255))
            self.image.blit(text, (int(10*lines/5)+30,20+int(40*lines/5) + i * 30))
        if not self._visible:
            self.image.fill((255, 255, 255, 0), None, pygame.BLEND_RGBA_MULT)        
        self.rect = self.image.get_rect()
        camera_offset = vec(global_var.SCREEN.size) - vec(global_var.CAMERA.size)
        if not flipx:
            if self._parent._gameObject.sprite:
                self.rect.center = Cartesian2pygame(self._parent.pos) - camera_offset//2 \
                    + 0.5 * vec(self._parent._gameObject.sprite.width, -self._parent._gameObject.sprite.height)+vec(30,-50)
            else:
                self.rect.center = Cartesian2pygame(self._parent.pos) - camera_offset//2 +vec(30,-50)                
        else:
            if self._parent._gameObject.sprite:
                self.rect.center = Cartesian2pygame(self._parent.pos) - camera_offset//2 \
                    - 0.5 * vec(self._parent._gameObject.sprite.width, self._parent._gameObject.sprite.height) +vec(-30,-50)
            else:
                self.rect.center = Cartesian2pygame(self._parent.pos) - camera_offset//2 +vec(-30,-50)
        global_var.ALL_SPRITES.move_to_front(self)

class TextBox(pygame.sprite.DirtySprite):
    def __init__(self, size = 20, font = './xiaobanma/simkai.ttf'):
        pygame.sprite.DirtySprite.__init__(self, global_var.ALL_SPRITES)
        self.font = pygame.font.Font(font, size)
        self._text = ''
        self._pos = vec(0, 0)
        self._color = (0, 0, 0)

    def print(self, text = None):
        self._text = str(text)

    def goto(self, x, *y):
        if y:
            self.pos = (x, *y)
        else:
            self.pos = x
    @ property
    def pos(self):
        return self._pos

    @ pos.setter
    def pos(self, pos):
        self._pos = pos

    @ property
    def color(self):
        return self._color

    @ color.setter
    def color(self, color):
        self._color = color

    def update(self):
        self.image = self.font.render(f"{self._text}", 2, self._color)
        self.rect = self.image.get_rect()
        self.rect.center = Cartesian2pygame(self._pos)
        global_var.ALL_SPRITES.move_to_front(self)



from .config import *
from .camera import Camera
from .physics import *

def setup(*size):
    if (not isinstance(size[0], int)) or (not isinstance(size[1], int)):
        raise TypeError('类型错误:参数必须为整数')
    if len(size) != 2:
        raise TypeError('类型错误:参数数目错误，只允许两个参数')
    global_var.SCREEN = Screen(size)
    global_var.WIDTH, global_var.HEIGHT = size
    global_var.ALL_SPRITES = pygame.sprite.LayeredDirty()
    global_var.CAMERA = Camera(size)
    global_var.SPACE = pymunk.Space()
    global_var.SPACE.gravity = (0, 0)
    global_var.BODIES = BodiesGroup()
    global_var.TMBG = TiledMapBodiesGroup()
    return global_var.SCREEN


def title(title):
    pygame.display.set_caption(title)

def update():
    global_var.SCREEN.update()
    global_var.JUST_RELEASED = False

def done():
    while True:
        global_var.SCREEN.update()

class Screen():
    def __init__(self, size):
        self.size = size
        self._screen = pygame.display.set_mode(size)
        self._clock = pygame.time.Clock()
 

    def update(self):
        global_var.EVENTS =  pygame.event.get()
        for event in global_var.EVENTS:
            if event.type == pygame.QUIT:
                pygame.quit()
                os._exit(0)
        global_var.GROUP.update()
        global_var.BODIES.update()   
        global_var.TMBG.update()     
        global_var.SPACE.step(1/60)
        rects = global_var.CAMERA.update()
        size = vec(global_var.CAMERA.size) * global_var.CAMERA.scl
        self._screen.blit(pygame.transform.scale(global_var.CAMERA.surface, [int(i) for i in size]), \
            global_var.CAMERA.offset + (vec(self.size) - size)//2)

        self.draw()
        pygame.display.update()
        self._clock.tick(40)
        self._screen.fill(BLACK)

    def draw(self):
        for p1, p2, color, width in global_var.LINES:
            pygame.draw.lines(self._screen, color, 0, [p1, p2], width)
        global_var.LINES = []

        








from .config import *
import pytmx
from pytmx.util_pygame import load_pygame
from .camera import Camera



class TiledMap():
    def __init__(self, tilemap):
        tm = load_pygame(tilemap, pixelalpha = False)
        self.width = tm.width * tm.tilewidth
        self.height = tm.height * tm.tileheight
        self._tmxdata = tm


    def render(self, surface):
        ti = self._tmxdata.get_tile_image_by_gid
        for layer in self._tmxdata.visible_layers:
            if isinstance(layer, pytmx.TiledTileLayer):
                for x, y, gid in layer:
                    tile = ti(gid)
                    if tile:
                        surface.blit(tile, (x * self._tmxdata.tilewidth, y * self._tmxdata.tileheight))


    def make_map(self):
        temp_surface = pygame.Surface((self.width, self.height)).convert_alpha()
        temp_surface.fill((0,0,0,0))
        self.render(temp_surface)
        return temp_surface

    @ property
    def objects(self):
        objects = []
        for obj in self._tmxdata.objects:
            obj.x = obj.x - (self.width - obj.width) // 2
            obj.y = (self.height - obj.height) // 2 - obj.y
            objects.append(obj)
        return objects

    def get_objects(self, name = None):
        from .API import NewGameObject
        from .game_object import GameObject
        from .physics import Body, TiledMapBodies
        from .textbox import DialogBox
        _objects = []
        for obj in self.objects:
            if obj.name == name or name == None:
                if 'points' in obj.__dict__:
                    points = []
                    for x, y in obj.points:
                        x = x - self.width // 2
                        y = self.height // 2 - y       
                        points.append((x, y))  
                    _objects.append(('POLY', points))
                else:
                    _objects.append(('BOX', (obj.width, obj.height), (obj.x, obj.y)))
        return _objects



from .config import *

class GameObject():
    def __init__(self):
        self._sprite = None
        self._body = None
        self._tiledmap_bodies = None
        self._pos = vec(0, 0)
        self._rot = 0
        self._scl = vec(1, 1)

    def __repr__(self):
        return f"Sprite;Physics"

    def set_sprite(self, sprite):
        self._sprite = sprite
        self._sprite.set_parent(self)

    def set_body(self, body):
        self._body = body
        if self._body:
            self._body.set_parent(self)

    def set_tiledmap_bodies(self, tiledmap_bodies):
        self._tiledmap_bodies = tiledmap_bodies
        if self._tiledmap_bodies:
            self._tiledmap_bodies.set_parent(self)

    @property
    def sprite(self):
        return self._sprite

    @property
    def body(self):
        return self._body
    
    @ property
    def tiledmap_bodies(self):
        return self._tiledmap_bodies

    @ property
    def pos(self):
        return self._pos

    @ pos.setter
    def pos(self, pos):
        if self._sprite:
            self._pos = vec(pos)
        if self._body:
            self._body.set_pos(pos)
        if self._tiledmap_bodies:
            self._tiledmap_bodies.set_pos(pos)

            
    @ property
    def x(self):
        return self._pos[0]

    @ property
    def y(self):
        return self._pos[1]

    @ x.setter
    def x(self, x):
        pos = vec(x, self._pos[1])
        self.pos = pos

    @ y.setter
    def y(self, y):
        pos = vec(self._pos[0], y)
        self.pos = pos

    @ property
    def rot(self):
        return self._rot

    @ rot.setter
    def rot(self, rot):
        if self.sprite:
            self.sprite.rotated = True
        self._rot = rot

    @ property
    def scl(self):
        return self._scl

    @ scl.setter
    def scl(self, xscl, *yscl):
        if yscl:
            self._scl = vec(xscl, *yscl)
        else:
            self._scl = vec(xscl)

    def kill(self):
        if self._sprite:
            self._sprite.kill()
            self._sprite
        if self._body:
            self._body.kill()
        if self._tiledmap_bodies:
           self._tiledmap_bodies.kill()
        self._sprite = None
        self._body = None
        self._tiledmap_bodies = None

    


from .config import *

def camera_offset(p):
    o = vec(global_var.SCREEN.size) - vec(global_var.CAMERA.size)
    newp = p - o //2
    newp = (int(newp[0]), int(newp[1]))   
    return newp 

class NewDrawOptions(pymunk.SpaceDebugDrawOptions):
    def __init__(self, surface):
        self.surface = surface
        super(NewDrawOptions, self).__init__()


    def draw_circle(self, pos, angle, radius, outline_color, fill_color):
        p = pymunk2Cartesian(pos, self.surface)
        pygame.draw.circle(self.surface, fill_color, p, _rndint(radius), 0)
        
        circle_edge = pos + Vec2d(radius, 0).rotated(angle)
        p2 = pymunk2Cartesian(circle_edge, self.surface)
        line_r = 2 if radius > 20 else 1
        pygame.draw.lines(self.surface, outline_color, False, [p,p2], line_r)    


    def draw_segment(self, a, b, color):
        p1 = pymunk2Cartesian(a, self.surface)
        p2 = pymunk2Cartesian(b, self.surface)

        pygame.draw.aalines(self.surface, color, False, [p1,p2])


    def draw_fat_segment(self, a, b, radius, outline_color, fill_color):
        p1 = pymunk2Cartesian(a, self.surface)
        p2 = pymunk2Cartesian(b, self.surface)
        
        r = _rndint(max(1, radius*2))
        pygame.draw.lines(self.surface, fill_color, False, [p1,p2], r)
        if r > 2:
            delta = ( p2[0]-p1[0], p2[1]-p1[1] )
            orthog = [ delta[1], -delta[0] ]
            scale = radius / (orthog[0]*orthog[0] + orthog[1]*orthog[1])**0.5
            orthog[0]*=scale; orthog[1]*=scale
            points = [
                ( p1[0]-orthog[0], p1[1]-orthog[1] ),
                ( p1[0]+orthog[0], p1[1]+orthog[1] ),
                ( p2[0]+orthog[0], p2[1]+orthog[1] ),
                ( p2[0]-orthog[0], p2[1]-orthog[1] )
            ]
            pygame.draw.polygon(self.surface, fill_color, points)
            pygame.draw.circle(self.surface, fill_color, 
                (_rndint(p1[0]),_rndint(p1[1])), _rndint(radius))
            pygame.draw.circle(self.surface, fill_color, 
                (_rndint(p2[0]),_rndint(p2[1])), _rndint(radius))

        
    def draw_polygon(self, verts, radius, outline_color, fill_color):
        ps = [pymunk2Cartesian(v, self.surface) for v in verts]
        ps += [ps[0]]

        pygame.draw.polygon(self.surface, fill_color, ps)

        if radius > 0:
            for i in range(len(verts)):
                a = verts[i]
                b = verts[(i+1) % len(verts)]
                self.draw_fat_segment(a, b, radius, outline_color, 
                    outline_color)


    def draw_dot(self, size, pos, color):
        p = pymunk2Cartesian(pos, self.surface)
        pygame.draw.circle(self.surface, color, p, _rndint(size), 0)

def pymunk2Cartesian(p, surface):
    return camera_offset((int(p[0] + global_var.WIDTH // 2), int(global_var.HEIGHT // 2 - p[1])))

def from_pygame(p, surface):
    """Convenience method to convert pygame surface local coordinates to 
    pymunk coordinates    
    """
    return to_pygame(p,surface)


def _rndint(x): 
    return int(round(x))      


from .config import *

class Spring():
    def __init__(self):
        pass

class Connect(pymunk.constraint.PinJoint):
    def __init__(self, a, b):
        pymunk.constraint.PinJoint.__init__(a._gameObject.body.body, b._gameObject.body.body, (0,0), (0, 0))
        global_var.SPACE.add(self) 

def connect(a, b):
    for vertice1 in a.vertices:
        for vertice2 in b.vertices:
            #global_var.SPACE.add(pymunk.constraint.PinJoint(a.body, b.body, vertice1, vertice2))
            global_var.SPACE.add(pymunk.constraint.GearJoint(a.body, b.body, 0, 1))
            pass






from .config import *
from .tiledmap import TiledMap

class EasySpriteStrategy():
    ''' 
    这是一个动画精灵类
    输入：一张图片名称
    方法：调用update方法来输出图片
    '''
    def __init__(self, img):
        if isinstance(img, str):
            self._image = pygame.image.load(img).convert_alpha()
        else:
            self._image = img

    def __len__(self):
        return 1

    def __getitem__(self, frame):
        raise NotImplementedError

    def __setitem__(self, frame):
        raise NotImplementedError
    
    def update(self):
        return self._image

class TiledMapStrategy():
    def __init__(self, tiledmap):
        self.tm = tiledmap.make_map().convert_alpha()


    def update(self):
        return self.tm


class ListSpriteStrategy():
    ''' 
    这是一个动画精灵类
    输入：以列表形式存储的图片名称
    方法：调用update方法来输出图片
    '''
    def __init__(self, imgs):
        self._frame = 0
        self._images = []
        self.playing = False
        self.dt = 0.05
        self._last_update_time = 0
        for img in imgs:
            if isinstance(img, str):
                self._images.append(pygame.image.load(img).convert_alpha())
            else:
                self._images.append(img)
    def __repr__(self):
        return f"sprites = {self._images}, frame = {self._frame}"

    def __len__(self):
        return len(self._images)

    def __getitem__(self, frame):
        assert frame < len(self._images), 'IndexError'
        return self.images[frame]

    def __setitem__(self):
        raise NotImplementedError

    @ property
    def frame(self):
        return self._frame

    @ frame.setter
    def frame(self, frame):
        self._frame = frame % len(self._images)

    def set_dt(self, dt, state):
        self.dt = dt
    
    def update(self):
        image = self._images[self._frame]
        if self.playing and time.time() - self._last_update_time > self.dt:
            self._last_update_time = time.time()
            self._frame += 1
            if self._frame == len(self._images):
                self._frame = 0
        return image

class AnimatorStrategy():
    ''' 
    这是一个动画精灵类
    输入：以字典形式存储的图片名称
    方法：调用update方法来输出图片
    '''
    def __init__(self, imgs_dict):
        self._frame = 0
        self._imgs_dict = {}
        self.playing = False
        self._last_update_time = 0
        self._state = 0
        for state in imgs_dict:
            self._imgs_dict[state] = {'animation':[],
                                      'dt':0.05,
                                      'next_state':state}
            for img in imgs_dict[state]:
                if isinstance(img, str):
                    self._imgs_dict[state]['animation'].append(pygame.image.load(img).convert_alpha())
                else:
                    self._imgs_dict[state]['animation'].append(img)
        self._state = state
        self._prestate = self._state
        self._state_temp = self._state

    def __repr__(self):
        return f"sprites = {self._imgs_dict}, frame = {self._frame}, state = {self._state}"

    def __len__(self):
        raise NotImplementedError

    def __getitem__(self, frame):
        raise NotImplementedError

    @ property
    def frame(self):
        return self._frame

    @ frame.setter
    def frame(self, frame):
        self._frame = frame % len(self._images)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, state):
        if state not in self._imgs_dict:
            raise KeyError
        self._state_temp = state

    def set_dt(self, dt, state = None):
        if state:
            self._imgs_dict[state]['dt'] = dt
        else:
            for state in self._imgs_dict:
                self._imgs_dict[state]['dt'] = dt

    def set_next_state(self, state, next_state):
        self._imgs_dict[state]['next_state'] = next_state

    def update(self):
        self._prestate = self._state
        self._state = self._state_temp
        if self._state != self._prestate:
            self._frame = 0
        image = self._imgs_dict[self._state]['animation'][self._frame]
        
        if self.playing and time.time() - self._last_update_time > self._imgs_dict[self._state]['dt']:
            self._last_update_time = time.time()
            self._frame += 1
            if self._frame == len(self._imgs_dict[self._state]['animation']):
                self.state = self._imgs_dict[self._state]['next_state']
                self._frame = 0
        return image


class SpriteSheet():
    def __init__(self, sheet, w, h):
        self.sheet = pygame.image.load(sheet).convert_alpha()
        self.imgs = []
        width, height = self.sheet.get_size()
        tile_width = width // w
        tile_height = height // h
        for i in range(h):
            for j in range(w):
                self.imgs.append(self.sheet.subsurface(pygame.Rect(j*tile_width, i*tile_height, tile_width, tile_height)))

    def __getitem__(self, key):
        return self.imgs[key]

    def __repr__(self):
        return self.imgs

class Sprite(pygame.sprite.DirtySprite):
    '''
    Only allow one image
    '''
    
    autos = {pygame.Surface:EasySpriteStrategy,
             str:EasySpriteStrategy,
             TiledMap:TiledMapStrategy,
             list:ListSpriteStrategy,
             dict:AnimatorStrategy,
             SpriteSheet:ListSpriteStrategy}
    def __init__(self, imgs):
        
        pygame.sprite.DirtySprite.__init__(self, global_var.ALL_SPRITES)
        self.dirty = 2
        self._offset = vec(0, 0)
        self._is_flip_h = False
        self._is_flip_v = False
        sprite_strategy = Sprite.autos[type(imgs)]
        self._sprite_strategy = sprite_strategy(imgs)
        self._red = 255
        self._green = 255
        self._blue = 255
        self._alpha = 255
        self._visible = True
        self.rect = self._sprite_strategy.update().get_rect()
        # Flags to inform the class whether to perform the transformation
        self.fliped = False
        self.scaled = False
        self.rotated = False
        self.modified = False

    def set_parent(self, parent):
        self._parent = parent

    @property
    def sprite_strategy(self):
        return self._sprite_strategy

    @property
    def is_flip_h(self):
        return self._is_flip_h

    @is_flip_h.setter
    def is_flip_h(self, h):
        self._is_flip_h = h

    @property
    def is_flip_v(self):
        return self._is_flip_v

    @is_flip_v.setter
    def is_flip_v(self, v):
        self._is_flip_v = v

    @property
    def red(self):
        return self._red

    @red.setter
    def red(self, r):
        self._red = r

    @property
    def green(self):
        return self._green

    @green.setter
    def green(self, g):
        self._green = g

    @property
    def blue(self):
        return self._blue

    @blue.setter
    def blue(self, b):
        self._blue = b

    @property
    def alpha(self):
        return 255 - self._alpha

    @alpha.setter
    def alpha(self, a):
        self._alpha = a

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, v):
        self._visible = v 

    @property
    def width(self):
        return self.rect.size[0]

    @property
    def height(self):
        return self.rect.size[1]

    def update(self):
        #self.image = self._sprite_strategy.update().copy() # 去掉copy更加节省时间
        self.image = self._sprite_strategy.update()

        # 翻转
        if self.fliped:
            self.image = pygame.transform.flip(self.image, self._is_flip_h, self._is_flip_v)
        # 放缩
        if self.scaled:
            width, height = self.image.get_rect().size    
            self.image = pygame.transform.scale(self.image, (int(self._parent.scl[0]*width), int(self._parent.scl[1]*height)))  
        # 旋转  
        if self.rotated:    
            self.image = pygame.transform.rotate(self.image, self._parent.rot)
        if self.modified:
            if self._visible:
                self.image.fill((self._red, self._green, self._blue, self._alpha), None, pygame.BLEND_RGBA_MULT)
            else:
                self.image.fill((self._red, self._green, self._blue, 0), None, pygame.BLEND_RGBA_MULT)

        self.rect = self.image.get_rect()
        camera_offset = vec(global_var.SCREEN.size) - vec(global_var.CAMERA.size)
        self.rect.center = Cartesian2pygame(self._parent.pos) - camera_offset//2 # 转化为笛卡尔坐标系




    




from abc import ABCMeta, abstractmethod
import pygame
from pygame.locals import *
vec = pygame.math.Vector2

class AbsStrategy(metaclass = ABCMeta):
    '''
    Return the image property to the sprite
    '''
    @abstractmethod
    def update(self, position, rotation, scale):
        pass

class AbsSprite(metaclass = ABCMeta):
    @abstractmethod
    def update(self):
        pass



class AbsGameObject(metaclass = ABCMeta):

    def __init__(self, sprite_strategy, physics_strategy):
        self._sprite_strategy = sprite_strategy()
        self._physics_strategy = physics_strategy()
        self._position = vec2(0, 0)
        self._rotation = 0
        self._scale = vec2(0, 0)

    def update(self):
        self._physics_strategy.update(self._position, self._rotation, self._scale)
        self._sprite_strategy.update(self._position, self._rotation, self._scale)

    @abstractmethod
    def next_image(self):
        pass

    @abstractmethod
    def set_image(self, frame):
        pass

    @abstractmethod
    def forward(self, d):
        pass

    @abstractmethod
    def backward(self, d):
        pass

    @abstractmethod
    def left(self, a):
        pass

    @abstractmethod
    def right(self, a):
        pass

    @abstractmethod
    def goto(self, x, *y):
        pass

    @abstractmethod
    def slide_to(self, x, *y):
        pass

    @abstractmethod
    def face_to(self, x, *y):
        pass



from .game_object import *
from .screen import *
from .sprite import *
from .physics import *
from .events import *
from .tiledmap import TiledMap
from .textbox import *
from .music import *
from .draw import *
from .constraints import *
import random

IS_UPDATED = 0
SPEED = 1

def tracer(f):
    global IS_UPDATED
    IS_UPDATED = f

def speed(s):
    global SPEED
    SPEED = s

def random_pos():
    return (random.randint(- global_var.WIDTH // 2, global_var.WIDTH // 2), random.randint(- global_var.HEIGHT // 2, global_var.HEIGHT // 2))

def debug():
    global_var.DEBUG_DRAW = True

def set_gravity(x, *y):
    if y:
        global_var.SPACE.gravity = (x, *y)
    else:
        global_var.SPACE.gravity = x

def set_mouse_visible(v):
    pygame.mouse.set_visible(v)

def sign(n):
    if n > 0:
        return 1
    elif n < 0:
        return -1
    else:
        return 0

class NewGameObject():
    def __init__(self, imgs = None, size = None, body_type = pymunk.Body.KINEMATIC, sensor = False):
        
        self._gameObject = GameObject()
        if imgs:
            self._gameObject.set_sprite(Sprite(imgs))
        if size:
            if isinstance(size, int):
                self._gameObject.set_body(Body(body_type, 'CIRCLE', size = size, sensor = sensor))
                self.radius = self._gameObject.body.shape.radius
            elif isinstance(size, tuple):
                self._gameObject.set_body(Body(body_type, 'BOX', size = size, sensor = sensor))
                self.vertices = self._gameObject.body.shape.get_vertices()
            elif isinstance(size, list):
                if isinstance(size[0][0], int):
                    self._gameObject.set_body(Body(body_type, 'POLY', size = size, sensor = sensor))
                    self.vertices = self._gameObject.body.shape.get_vertices()  
                else:
                    tiledmap_objects = TiledMapBodies(body_type, size, sensor)
                    self._gameObject.set_tiledmap_bodies(tiledmap_objects)


                          
        self._direction = vec(1, 0)
        self._dialogBox = None
        self._get_clicked = False

        global_var.GROUP.add(self)
        self._just_released = False
        self.collision_dict = {}
        if self._gameObject.sprite:
            self._gameObject.sprite.update()     

    def __len__(self):
        raise NotImplementedError

    def __getitem__(self):
        raise NotImplementedError

    def __setitem__(self):
        raise NotImplementedError

    def __bool__(self):
        return True

    def __del__(self):
        self.kill()

    '''
    运动命令
    '''
    def forward(self, d):
        n = 0
        while n < abs(d):
            self._gameObject.pos += self._direction * SPEED * sign(d)
            n += SPEED
            if IS_UPDATED:
                update()

    def backward(self, d):
        n = 0
        while n < abs(d):
            self._gameObject.pos -= self._direction * SPEED * sign(d)
            n += SPEED
            if IS_UPDATED:
                update()

    def _rotate(self, rot):
        self.rotate_image(rot)
        self.rotate_direction(rot) 

    def rotate_direction(self, rot):
        self.dir = self.dir.rotate(rot)

    def rotate_image(self, rot):
        self.rot += rot

    @ property
    def rot(self):
        return self._gameObject.rot

    @ rot.setter
    def rot(self, r):
        self._gameObject.rot = r
        self._gameObject.sprite.rotated = True
        if self._gameObject.body:
            self._gameObject.body.set_rot(r)
        if self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.set_rot(r)


    @ property
    def dir(self):
        return self._direction

    @ dir.setter
    def dir(self, d):
        self._direction = d

    def left(self, a): 
        n = 0
        speed = min(a, SPEED)
        while n < abs(a):
            self._rotate(speed * sign(a))
            n += speed
            if IS_UPDATED:
                update()

    def right(self, a):
        n = 0
        speed = min(a, SPEED)
        while n < abs(a):
            self._rotate(-speed * sign(a))
            n += speed
            if IS_UPDATED:
                update()

    def slide_to(self, x, *y):
        if y:
            distance = vec(x, *y) - self._gameObject.pos
            if distance.length() < 1:
                self._gameObject.pos = vec(x, *y)
                return True
        else:
            distance = vec(x) - self._gameObject.pos
            if distance.length() < 1:
                self._gameObject.pos = vec(x)
                return True
        self._gameObject.pos += distance.normalize() * SPEED
        return False

    def goto(self, x, *y):
        if y:
            self._gameObject.pos = vec(x, *y)
        else:
            self._gameObject.pos = vec(x)
        if self._gameObject.body:
            self._gameObject.body.velocity = vec(0,0)
        #update()

    def face_to(self, x, *y, VEC = False):
        if not VEC:
            if y:
                self.rot = - (vec(x, *y) - self._gameObject.pos).angle_to(vec(1, 0)) 
                self.dir = (vec(x, *y) - self._gameObject.pos).normalize()
            else:
                self.rot = - (vec(x) - self._gameObject.pos).angle_to(vec(1, 0))  
                if  (vec(x) - self._gameObject.pos).length() != 0:
                    self.dir = (vec(x) - self._gameObject.pos).normalize()   
        else:
            if y:
                self.rot = - vec(x, *y).angle_to(vec(1, 0)) 
                self.dir = vec(x, *y).normalize()
            else:
                self.rot = - vec(x).angle_to(vec(1, 0))  
                if  vec(x).length() != 0:
                    self.dir = vec(x).normalize()               

    @ property
    def pos(self):
        return self._gameObject.pos

    @ property
    def dir(self):
        return self._direction
        
    @ property
    def x(self):
        return self._gameObject.pos[0]

    @ property
    def y(self):
        return self._gameObject.pos[1]

    @ pos.setter
    def pos(self,p):
        self._gameObject.pos = p
        if self._gameObject.body:
            self._gameObject.body.velocity = vec(0,0)
        if self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.velocity = vec(0,0)        
        

    @ dir.setter
    def dir(self, dir):
        self._direction = dir

    @ x.setter
    def x(self, x):
        if self._gameObject.body:
            self._gameObject.body.velocity = vec(0,0)
        if self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.velocity = vec(0,0)
        self._gameObject.x = x

    @ y.setter
    def y(self, y):
        if self._gameObject.body:
            self._gameObject.body.velocity = vec(0,0)
        if self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.velocity = vec(0,0)
        self._gameObject.y = y

    @ property
    def velocity(self):
        if self._gameObject.body:
            return self._gameObject.body.velocity
        elif self._gameObject.tiledmap_bodies:
            return self._gameObject.tiledmap_bodies.velocity

    @ velocity.setter
    def velocity(self, v):
        if self._gameObject.body:
            self._gameObject.body.velocity = v
        elif self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.velocity = v

    @ property
    def angular_velocity(self):
        if self._gameObject.body:
            return self._gameObject.body.angular_velocity
        elif self._gameObject.tiledmap_bodies:
            return self._gameObject.tiledmap_bodies.angular_velocity
            
    @ angular_velocity.setter
    def angular_velocity(self, v):
        if self._gameObject.body:
            self._gameObject.body.angular_velocity = v
        elif self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.angular_velocity = v
    '''
    图片命令
    '''
    def scale(self, scalex, scaley = None):
        if scaley == None:
            if self._gameObject.sprite:
                self._gameObject.sprite.scaled = True
                self._gameObject.scl = scalex
            if self._gameObject.body:
                self.body_scale(scalex, scaley)
            if self._gameObject.tiledmap_bodies:
                self.body_scale(scalex, scaley)
        else:
            if self._gameObject.sprite:
                self._gameObject.sprite.scaled = True
                self._gameObject.scl = (scalex, scaley)
            if self._gameObject.body:
                self.body_scale(scalex, scaley)            
            if self._gameObject.tiledmap_bodies:
                self.body_scale(scalex, scaley)            
            

    def flipx(self, h):
        self._gameObject.sprite.fliped = True
        self._gameObject.sprite.is_flip_h = h

    def flipy(self, v):
        self._gameObject.sprite.fliped = True
        self._gameObject.sprite.is_flip_v = v

    def set_color_mask(self, red, green, blue, alpha = 0):
        # 不建议对过大的地图进行此操作，会导致动画变慢
        self._gameObject.sprite.modified = True
        self._gameObject.sprite.red = red
        self._gameObject.sprite.green = green
        self._gameObject.sprite.blue = blue
        self._gameObject.sprite.alpha = alpha

    def show(self):
        # 不建议对过大的地图进行此操作，会导致动画变慢
        self._gameObject.sprite.modified = True
        self._gameObject.sprite.visible = True

    def hide(self):
        # 不建议对过大的地图进行此操作，会导致动画变慢
        self._gameObject.sprite.modified = True
        self._gameObject.sprite.visible = False
    
    @ property
    def visible(self):
        return self._gameObject.sprite.visible

    @ property
    def width(self):
        return self._gameObject.sprite.width

    @ property
    def height(self):
        return self._gameObject.sprite.height

    # 动画命令
    def next_image(self):
        self._gameObject.sprite.sprite_strategy.frame += 1

    def set_image(self, f):
        self._gameObject.sprite.sprite_strategy.frame = f

    def play_anim(self):
        self._gameObject.sprite.sprite_strategy.playing = True

    @ property
    def state(self):
        return self._gameObject.sprite.sprite_strategy.state

    @ state.setter
    def state(self, state):
        self._gameObject.sprite.sprite_strategy.state = state

    def set_dt(self, dt, state = None):
        self._gameObject.sprite.sprite_strategy.set_dt(dt, state)

    @ property
    def dt(self):
        if isinstance(self._gameObject.sprite.sprite_strategy, ListSpriteStrategy):
            return self._gameObject.sprite.sprite_strategy.dt
        else:
            raise NotImplementedError

    @ dt.setter
    def dt(self, dt):
        self.set_dt(dt)


    def set_next_state(self, state, next_state):
        self._gameObject.sprite.sprite_strategy.set_next_state(state, next_state)

    # 图层
    @ property
    def layer(self):
        return global_var.ALL_SPRITES.get_layer_of_sprite(self._gameObject.sprite)

    @ layer.setter
    def layer(self, layer):
        global_var.ALL_SPRITES.change_layer(self._gameObject.sprite, layer)

    def move_to_front(self):
        global_var.ALL_SPRITES.move_to_front(self._gameObject.sprite)

    def move_to_back(self):
        global_var.ALL_SPRITES.move_to_back(self._gameObject.sprite)    

    # 音效
    def play_snd(self, snd = None, v=1):
        if snd:
            self.snd = pygame.mixer.Sound(snd)
            self.snd.set_volume(v)
            self.snd.stop()
            self.snd.play() 
        elif snd == '':
            pass
        else:
            if 'snd' in self.__dict__:
                self.snd.stop()   
        self.snd = snd
        
    # 交互
    
    def get_mouse_clicked(self):
        # 检测鼠标左键是否在按下的状态下
        for event in global_var.EVENTS:
            if event and event.type == MOUSEBUTTONDOWN and self.get_mouse_upon():
                self._get_clicked = True
        self.get_mouse_just_released()
        if self._get_clicked:
            return True
        else:
            return False
    
    def get_mouse_just_clicked(self):
        # 检测鼠标是否刚刚被按下
        for event in global_var.EVENTS:
            if event and event.type == MOUSEBUTTONDOWN and self.get_mouse_upon():
                self._get_clicked = True
                return True
        else:
            return False

    def get_mouse_just_released(self):
        # 检测鼠标是否刚刚被松开
        if self._just_released:
            return True
        self.get_mouse_just_clicked()
        if self._get_clicked:
            for event in global_var.EVENTS:
                if event and event.type == MOUSEBUTTONUP:
                    self._get_clicked = False
                    self._just_released = True
            if self._get_clicked:
                return False
            else:
                return True
        else:
            return False

    def get_mouse_upon(self):
        # 检测鼠标是否在图片上
        if self.collide(get_mouse_pos()):
            return True
        else:
            return False

    

    # 其他
    def collide(self, other):
        if self._gameObject.sprite and self._gameObject.sprite.visible:
            if isinstance(other, tuple) or isinstance(other, vec):
                if self._gameObject.body:  
                    return self._gameObject.body.point_query(other)
                elif self._gameObject.tiledmap_bodies:
                    return self._gameObject.tiledmap_bodies.point_query(other)
                else:
                    mask = pygame.mask.from_surface(self._gameObject.sprite.image)
                    offset = self._gameObject.pos - other \
                        + vec(self._gameObject.sprite.width//2, self._gameObject.sprite.height//2)
                    try:
                        return bool(mask.get_at([int(i) for i in offset]))
                    except:
                        return False
            else:         
                if self._gameObject.body:       
                    return self._gameObject.body.collide(other._gameObject.body)
                elif self._gameObject.tiledmap_bodies:
                    if other._gameObject.body:    
                        return self._gameObject.tiledmap_bodies.collide(other._gameObject.body)
                    elif other._gameObject.tiledmap_bodies:    
                        return self._gameObject.tiledmap_bodies.collide(other._gameObject.tiledmap_bodies)
                else:
                    mask1 = pygame.mask.from_surface(self._gameObject.sprite.image)
                    mask2 = pygame.mask.from_surface(other._gameObject.sprite.image)
                    offset = self._gameObject.pos - other._gameObject.pos \
                        + vec(self._gameObject.sprite.width//2, self._gameObject.sprite.height//2)\
                        - vec(other._gameObject.sprite.width//2, other._gameObject.sprite.height//2)
                    return bool(mask1.overlap(mask2, [int(i) for i in offset]))
        elif not self._gameObject.sprite:
            if isinstance(other, tuple) or isinstance(other, vec):
                if self._gameObject.body:  
                    return self._gameObject.body.point_query(other)
                elif self._gameObject.tiledmap_bodies:
                    return self._gameObject.tiledmap_bodies.point_query(other)
            else:         
                if self._gameObject.body:       
                    return self._gameObject.body.collide(other._gameObject.body)
                elif self._gameObject.tiledmap_bodies:
                    if other._gameObject.body:    
                        return self._gameObject.tiledmap_bodies.collide(other._gameObject.body)
                    elif other._gameObject.tiledmap_bodies:    
                        return self._gameObject.tiledmap_bodies.collide(other._gameObject.tiledmap_bodies)
                
        else:
            return False

    def separate(self, other): 
        # 不要检测是否与get_mouse_pos()函数的分离，因为只能检测和固定点之间的分离
        if other not in self.collision_dict:
            self.collision_dict[other] = False
        if not self.collide(other) and self.collision_dict[other]:
            self.collision_dict[other] = False
            return True
        else:
            if self.collide(other):
                self.collision_dict[other] = True
            else:
                self.collision_dict[other] = False
            return False


    def distance(self, other):
        return self._gameObject.pos.distance_to(other._gameObject.pos)

    def say(self, text = None):
        if not self._dialogBox:
            self._dialogBox = DialogBox(self)
        if text:
            self._dialogBox.show()
            self._dialogBox.say(text)
        else:
            self._dialogBox.hide()
        if IS_UPDATED:
            update()

    def kill(self):
        self._gameObject.kill()
        if self._dialogBox:
            self._dialogBox.kill()

    def apply_force(self, x, *y):
        if self._gameObject.body:
            if y:
                self._gameObject.body.apply_impulse_at_local_point((x, *y))
            else:
                self._gameObject.body.apply_impulse_at_local_point(x)
        elif self._gameObject.tiledmap_bodies:
            if y:
                self._gameObject.tiledmap_bodies.apply_impulse_at_local_point((x, *y))
            else:
                self._gameObject.tiledmap_bodies.apply_impulse_at_local_point(x)            


    @ property
    def elasticity(self):
        if self._gameObject.body:
            return self._gameObject.body.elasticity
        elif self._gameObject.tiledmap_bodies:
            return self._gameObject.tiledmap_bodies.elasticity

    @ elasticity.setter
    def elasticity(self, e):
        if self._gameObject.body:
            self._gameObject.body.elasticity = e
        elif self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.elasticity = e


    @ property
    def friction(self):
        if self._gameObject.body:
            return self._gameObject.body.friction
        elif self._gameObject.tiledmap_bodies:
            return self._gameObject.tiledmap_bodies.friction

    @ friction.setter
    def friction(self, f):
        if self._gameObject.body:
            self._gameObject.body.friction = f 
        elif self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.friction = f 

    @ property
    def mass(self):
        if self._gameObject.body:
            return self._gameObject.body.mass
        elif self._gameObject.tiledmap_bodies:
            return self._gameObject.tiledmap_bodies.mass

    @ mass.setter
    def mass(self, m): 
        if self._gameObject.body:
            self._gameObject.body.mass = m
        elif self._gameObject.tiledmap_bodies:
            self._gameObject.tiledmap_bodies.mass = m

    def body_scale(self, scalex, scaley):
        if "radius" in self.__dict__:
            if scaley:
                print('圆形无法进行不同方向的拉伸')
            self._gameObject.body.shape.unsafe_set_radius(int(self.radius * scalex))
        elif "vertices" in self.__dict__:
            if scaley:
                self._gameObject.body.set_vertices(self.vertices, transform=pymunk.Transform(a = scalex, d = scaley))
            else:
                self._gameObject.body.set_vertices(self.vertices, transform=pymunk.Transform(a = scalex, d = scalex))
        elif self._gameObject.tiledmap_bodies:
            if scaley:
                self._gameObject.tiledmap_bodies.set_vertices(scalex, scaley)
            else:
                self._gameObject.tiledmap_bodies.set_vertices(scalex, scalex)
        else:
            raise NotImplementedError

    def _update(self):
        # 私有方法, 不要重载!!
        self._just_released = False

    def update(self):
        # 允许重载的方法
        pass
        
class Character(NewGameObject):
    '''
    允许通过代码控制其运动的类
    '''
    def __init__(self, imgs = None, size = None):
        # 可以通过给出一张图片或图片列表或来实现人物创建
        NewGameObject.__init__(self, imgs = imgs, size = size, body_type = "DYNAMIC", sensor = False)

class Sensor(NewGameObject):
    def __init__(self, imgs = None, size = None):
        # 可以通过给出一张图片或图片列表或来实现人物创建
        NewGameObject.__init__(self, imgs = imgs, size = size, body_type = "KINEMATIC", sensor = True)

class Wall(NewGameObject):
    '''
    墙壁,无法运动的物体
    '''
    def __init__(self, imgs = None, size = None):
        # 可以通过给出一张图片或图片列表或来实现人物创建
        NewGameObject.__init__(self, imgs = imgs, size = size, body_type = "STATIC", sensor = False)



class Mouse(Character):
    def __init__(self, imgs = None, radius = 1):
        Character.__init__(self, imgs, radius)
        if imgs:
            set_mouse_visible(False)

    def body_scale(self, scalex, scaley = None):
        pass
        
    def get_pos(self):
        return get_mouse_pos()

    def get_rel(self):
        return get_mouse_rel()

    def get_clicked(self):
        return get_mouse_clicked()

    def get_just_clicked(self):
        return get_mouse_just_clicked()

    def get_just_released(self):
        return get_mouse_just_released()

    def update(self):
        self.goto(get_mouse_pos())






def bgpic(img):
    bg = Character(img)
    if isinstance(bg, TiledMap):
        global_var.CAMERA = Camera((bg.width, bg.height))
    else:
        global_var.CAMERA = Camera((bg._gameObject.sprite.width, bg._gameObject.sprite.height))
    return bg


from .config import *
from .constraints import *


class BodiesGroup():
    def __init__(self):
        self._group = []

    def update(self):
        for body in self._group:
            body.update()

    def add(self, body):
        self._group.append(body)

    def remove(self, body):
        self._group.remove(body)

class TiledMapBodiesGroup():
    def __init__(self):
        self._group = []

    def update(self):
        for tiledmapbodies in self._group:
            tiledmapbodies.update()

    def add(self, tiledmapbodies):
        self._group.append(tiledmapbodies)

    def remove(self, tiledmapbodies):
        self._group.remove(tiledmapbodies)

class Body():
    def __init__(self, body_type, shape, size, sensor):
        mass = 1
        moment = 1
        if shape == 'CIRCLE':
            moment = pymunk.moment_for_circle(mass, 0, size)    
        if body_type == 'KINEMATIC':
            self.body = pymunk.Body(mass, moment, body_type = pymunk.Body.KINEMATIC)
        elif body_type == 'DYNAMIC':
            self.body = pymunk.Body(mass, moment, body_type = pymunk.Body.DYNAMIC)
        elif body_type == 'STATIC':
            self.body = pymunk.Body(mass, moment, body_type = pymunk.Body.STATIC)
        self.body.position = 0,0
        if shape == 'CIRCLE':
            self.shape = pymunk.Circle(self.body, size) 
        elif shape == 'BOX':
            width, height = size
            points = [(-width//2, -height//2), (-width//2, height//2), (width//2,height//2), (width//2, -height//2)]
            self.shape = pymunk.Poly(self.body, points)
        elif shape == 'POLY':
            self.shape = pymunk.Poly(self.body, size)
        self.shape.sensor = sensor
        global_var.SPACE.add(self.body, self.shape) 
        global_var.BODIES.add(self)


    def set_parent(self, parent):
        self._parent = parent
        if 'offset' in self.__dict__:
            self.shape.body.position = self._parent.pos + self.offset
        else:
            self.shape.body.position = self._parent.pos
            
    def set_pos(self, pos):
        self.shape.body.position = pos
        global_var.SPACE.reindex_shapes_for_body(self.body)

    def update(self):
        if 'offset' not in self.__dict__:
            self._parent.pos = self.shape.body.position
        self._parent.rot = self.body.angle / 3.1415926 * 180
        
    def collide(self, other):
        if self.shape.shapes_collide(other.shape).points:
            return True
        else:
            return False

    def point_query(self, p):
        if self.shape.point_query(p)[0] <= 0:
            return True
        else:
            return False

    def kill(self):
        global_var.SPACE.remove(self.body, self.shape)
        global_var.BODIES.remove(self)  

    def set_rot(self, rot):
        self.body.angle = rot / 180 * 3.1415926
        global_var.SPACE.reindex_shapes_for_body(self.body)       

    @ property
    def velocity(self):
        return self.body.velocity 

    @ velocity.setter
    def velocity(self, v):
        self.body.velocity = v

    @ property
    def angular_velocity(self):
        return self.body.angular_velocity

    @ angular_velocity.setter
    def angular_velocity(self, av):
        self.body.angular_velocity = av

    def apply_impulse_at_local_point(self, impulse):
        self.body.apply_impulse_at_local_point(impulse)

    @ property
    def elasticity(self):
        return self.shape.elasticity

    @ elasticity.setter
    def elasticity(self, e):
        self.shape.elasticity = e

    @ property
    def friction(self):
        return self.shape.friction

    @ friction.setter
    def friction(self, f):
        self.shape.friction = f  

    @ property
    def mass(self):
        return self.body.mass

    @ mass.setter
    def mass(self, m): 
        self.body.mass = m

    def set_vertices(self, vertices, transform):
        self.shape.unsafe_set_vertices(vertices, transform)

class TiledMapBodies(list):
    def __init__(self, body_type, sizes, sensor):

        list.__init__(self)
        global_var.TMBG.add(self)
        self._velocity = vec(0, 0)
        self._angular_velocity = 0
        self._mass = 1
        self._friction = 0
        self._elasticity = 1
        self.pos = vec(0, 0)
        for size in sizes:
            shape = size[0]
            if shape == 'POLY':
                points = size[1]
            else:
                width, height = size[1]
                x, y = size[2]
                points = [(x + width // 2, y + height // 2),
                          (x + width // 2, y - height // 2),
                          (x - width // 2, y + height // 2),
                          (x - width // 2, y - height // 2)]
            body = Body(body_type, 'POLY', points, sensor)
            body.offset = vec(0, 0)
            body.type = size[0]
            body.vertices = body.shape.get_vertices()
            body.offsetrot = 0
            body.offsetscale = vec(1, 1)
            body.set_parent(self)
            self.append(body)
        if body_type == 'DYNAMIC':
            print('不建议将网格地图设置为Character')
            dict = []
            for body in self:
                for other in self:
                    if body != other and {body, other} not in dict:
                        connect(body, other)
                        dict.append({body, other})

    def set_parent(self, parent):
        self._parent = parent
        self.pos = self._parent.pos
            
    def set_pos(self, pos):
        self.pos = pos          
        for body in self:
            body.offset_scaled = vec(body.offset[0] * body.offsetscale[0], body.offset[1] * body.offsetscale[1])
            body.body.position = self.pos + body.offset_scaled.rotate(body.offsetrot)
            global_var.SPACE.reindex_shapes_for_body(body.body)  
        
    def update(self):
        positions = vec(0,0)
        for body in self:
                positions += body.body.position
        positions /= len(self)
        self.pos = positions
        self._parent.pos = self.pos
        self._parent.rot = self[0].body.angle / 3.1415926 * 180
        

    def collide(self, other):
        for body in self:
            if body.shape.shapes_collide(other.shape).points:
                return True
        else:
            return False

    def point_query(self, p):
        for body in self:
            if body.shape.point_query(p)[0] <= 0:
                return True
        else:
            return False

    def kill(self):
        for body in self:
            global_var.SPACE.remove(body.body, body.shape)
            global_var.BODIES.remove(body) 

    def set_rot(self, rot):
        for body in self:
            body.body.angle = rot / 180 * 3.1415926  
            body.offsetrot = rot 
            global_var.SPACE.reindex_shapes_for_body(body.body)       

    @ property
    def velocity(self):
        return self._velocity 

    @ velocity.setter
    def velocity(self, v):
        self._velocity = v
        for body in self:
            body.body.velocity = v

    @ property
    def angular_velocity(self):
        return self._angular_velocity

    @ angular_velocity.setter
    def angular_velocity(self, av):
        self._angular_velocity = av
        for body in self:
            body.body.angular_velocity = av
        

    def apply_impulse_at_local_point(self, pos):
        for body in self:
            body.body.apply_impulse_at_local_point(pos)

    @ property
    def elasticity(self):
        return self._elasticity

    @ elasticity.setter
    def elasticity(self, e):
        self._elasticity = e
        for body in self:
            body.body.elasticity = e

    @ property
    def friction(self):
        return self._friction

    @ friction.setter
    def friction(self, f):
        self._friction = f  
        for body in self:
            body.body.friction = f

    @ property
    def mass(self):
        return self._mass

    @ mass.setter
    def mass(self, m): 
        self._mass = m
        for body in self:
            body.body.mass = m

    def set_vertices(self, scalex, scaley):
        for body in self:
            body.set_vertices(body.vertices, transform=pymunk.Transform(a = scalex, d = scaley))
            body.offsetscale = vec(scalex, scaley)
            


from .config import *
from .game_object import GameObject
from .event import get_mouse_pos
class Coordinate(Sensor):
    def __init__(self):
        Sensor.__init__(self, './img/cursor.png')
        self.text = TextBox(25)

    def update(self):
        super().update()
        self.move_to_front()
        self.goto(get_mouse_pos() + vec(120, 70))
        self.text.goto(get_mouse_pos() + vec(120, 70))
        self.text.print(get_mouse_pos())


import pygame
from pygame.locals import *
import os
from .settings import *
import pymunk
import pymunk.pygame_util
from pymunk.vec2d import Vec2d
import time
from .facilitate import *
vec = pygame.math.Vector2
pygame.mixer.pre_init(44100, -16, 2, 1024)
pygame.mixer.init()
pygame.init()

pygame.font.init()



cwd = os.getcwd()
class global_var:
    WIDTH = 0
    HEIGHT = 0
    EVENTS = []
    ALL_SPRITES = None
    ALL_BODIES = None
    SCREEN = None
    SPACE = None
    CAMERA = None
    DEBUG_DRAW = False
    GET_CLICKED = False
    JUST_RELEASED = False
    TMBG = None
    LINES = []

    @ classmethod
    def set_value(cls, var, value):
        cls.var = value

    @ classmethod
    def get_value(cls, var):
        return cls.var

def pygame2Cartesian(x, *y):
    if y:
        return (x - global_var.WIDTH // 2, global_var.HEIGHT // 2 - y)
    else:
        return (x[0] - global_var.WIDTH // 2, global_var.HEIGHT // 2 - x[1])

def Cartesian2pygame(x, *y):
    if y:
        return (x + global_var.WIDTH // 2, global_var.HEIGHT // 2 - y)
    else:
        return (x[0] + global_var.WIDTH // 2, global_var.HEIGHT // 2 - x[1])

class Group():
    def __init__(self):
        self._group = []

    def add(self, obj):
        self._group.append(obj)

    def update(self):
        for obj in self._group:
            obj._update()
            obj.update()
global_var.GROUP = Group()


from .config import *
from .draw_options import NewDrawOptions

class Camera():
    _instances = {}

    def __new__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__new__(cls)
            cls._instances[cls] = instance
        return cls._instances[cls]

    def __init__(self, size = None):
        if size:
            self.surface = pygame.Surface(size)
            self.scl = 1
            self._position = vec(0, 0)
            self._subject = None
            self.draw_options = NewDrawOptions(self.surface)
        else:
            self = global_var.CAMERA

    @ property
    def size(self):
        return self.surface.get_size()


    def update(self):       
        self.surface.fill((255, 255, 255))
        if self._subject:
            self._position = self._subject.pos

        global_var.ALL_SPRITES.update()
        rects = global_var.ALL_SPRITES.draw(self.surface)
        if global_var.DEBUG_DRAW:
            global_var.SPACE.debug_draw(self.draw_options) 

        #return rects

    def follow(self, subject):
        self._subject = subject

    def scale(self,s):
        self.scl = s

    @property
    def offset(self):
        return (-self._position[0], self._position[1])

    @ property
    def x(self):
        return self._position[0]

    @ property
    def y(self):
        return self._position[1]   

    @ property
    def pos(self):
        return self._position

    @ x.setter
    def x(self, x):
        self._position[0] = x

    @ y.setter
    def y(self, y):
        self._position[1] = y

    @ pos.setter
    def pos(self, pos):
        self._position = pos


import time
def time_count(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        res = func(*args, **kwargs)
        print(func, time.time() - start)
        return res
    return wrapper


from .config import *
#pygame.key.set_repeat(300, 100)
def get_mouse_pos():
    # 获得鼠标坐标
    return pygame2Cartesian(pygame.mouse.get_pos())


def get_mouse_rel():
    # 获得鼠标运动
    '''
    x, y = pygame.mouse.get_rel()
    return vec(x, -y)
    '''
    for event in global_var.EVENTS:
        if event.type == MOUSEMOTION:
            x, y = event.rel
            return vec(x, -y)
    else:
        return vec(0, 0)

def get_mouse_clicked():
    # 检测鼠标左键是否在按下的状态下
    for event in global_var.EVENTS:
        if event and event.type == MOUSEBUTTONDOWN:
            global_var.GET_CLICKED = True
    get_mouse_just_released()
    if global_var.GET_CLICKED:
        return True
    else:
        return False
    
def get_mouse_just_clicked():
    # 检测鼠标是否刚刚被按下
    for event in global_var.EVENTS:
        if event and event.type == MOUSEBUTTONDOWN:
            global_var.GET_CLICKED = True
            return True
    else:
        return False

def get_mouse_just_released():
    # 检测鼠标是否刚刚被松开
    if global_var.JUST_RELEASED:
        return True
    get_mouse_just_clicked()
    if global_var.GET_CLICKED:
        for event in global_var.EVENTS:
            if event and event.type == MOUSEBUTTONUP:
                global_var.GET_CLICKED = False
                global_var.JUST_RELEASED = True
        if global_var.GET_CLICKED:
            return False
        else:
            return True
    else:
        return False

def key_pressed(key = None):
    if not key:
        if 1 in pygame.key.get_pressed():
            return True
        else:
            return False
    if pygame.key.get_pressed()[key]:
        return True
    else:
        return False

def key_just_pressed(*keys):
    keyEvents = []
    for event in global_var.EVENTS:
        if event.type == KEYDOWN:
            keyEvents.append(event.key)
    if not keys:
        return keyEvents
    for key in keys:
        if key not in keyEvents:
            break
    else:
        return True
    return False


def key_released(*keys): 
    keyEvents = []
    for event in global_var.EVENTS:
        if event.type == KEYUP:
            keyEvents.append(event.key)
    if not keys:
        return keyEvents
    for key in keys:
        if key not in keyEvents:
            break
    else:
        return True
    return False


"""Very basic example of using a sprite image to draw a shape more similar 
how you would do it in a real game instead of the simple line drawings used 
by the other examples. 
"""

__version__ = "$Id:$"
__docformat__ = "reStructuredText"

import math, random

import pygame
from pygame.locals import *
from pygame.color import *

import pymunk
from pymunk import Vec2d

def flipy(y):
    """Small hack to convert chipmunk physics to pygame coordinates"""
    return -y+600

def main():
            
    pygame.init()
    screen = pygame.display.set_mode((600, 600))
    clock = pygame.time.Clock()
    running = True
    
    ### Physics stuff
    space = pymunk.Space()
    space.gravity = Vec2d(0.0, -900.0)
    
    ## logo
    logo_img = pygame.image.load("pymunk_logo_googlecode.png")
    logos = []
    
    
    ### Static line
    static_lines = [pymunk.Segment(space.static_body, (11.0, 280.0), (407.0, 246.0), 0.0)
                    ,pymunk.Segment(space.static_body, (407.0, 246.0), (407.0, 343.0), 0.0)
                    ]
    for l in static_lines:
        l.friction = 0.5
    space.add(static_lines)

    ticks_to_next_spawn = 10
    
    while running:
        for event in pygame.event.get():
            if event.type == QUIT:
                running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                running = False
            elif event.type == KEYDOWN and event.key == K_p:
                pygame.image.save(screen, "using_sprites.png")  
        
        ticks_to_next_spawn -= 1
        if ticks_to_next_spawn <= 0:
            ticks_to_next_spawn = 100
            x = random.randint(20,400)
            y = 500
            angle = random.random() * math.pi
            vs = [(-23,26), (23,26), (0,-26)]
            mass = 10
            moment = pymunk.moment_for_poly(mass, vs)
            body = pymunk.Body(mass, moment)
            shape = pymunk.Poly(body, vs)
            shape.friction = 0.5
            body.position = x, y
            body.angle = angle
            
            space.add(body, shape)
            logos.append(shape)
       
        ### Update physics
        dt = 1.0/60.0
        for x in range(1):
            space.step(dt)
            
        ### Draw stuff
        screen.fill(THECOLORS["black"])

        for logo_shape in logos:
            # image draw
            p = logo_shape.body.position
            p = Vec2d(p.x, flipy(p.y))
            
            # we need to rotate 180 degrees because of the y coordinate flip
            angle_degrees = math.degrees(logo_shape.body.angle) + 180 
            rotated_logo_img = pygame.transform.rotate(logo_img, angle_degrees)
            
            offset = Vec2d(rotated_logo_img.get_size()) / 2.
            p = p - offset
            
            screen.blit(rotated_logo_img, p)
            
            # debug draw
            ps = [p.rotated(logo_shape.body.angle) + logo_shape.body.position for p in logo_shape.get_vertices()]
            ps = [(p.x, flipy(p.y)) for p in ps]
            ps += [ps[0]]
            pygame.draw.lines(screen, THECOLORS["red"], False, ps, 1)
           

        for line in static_lines:
            body = line.body
            
            pv1 = body.position + line.a.rotated(body.angle)
            pv2 = body.position + line.b.rotated(body.angle)
            p1 = pv1.x, flipy(pv1.y)
            p2 = pv2.x, flipy(pv2.y)
            pygame.draw.lines(screen, THECOLORS["lightgray"], False, [p1,p2], 2)

        ### Flip screen
        pygame.display.flip()
        clock.tick(50)
        pygame.display.set_caption("fps: " + str(clock.get_fps()))
        
if __name__ == '__main__':
    main()